classdef CQGImpVolAndGreeks < handle
    
    % Example
    % >> CQGObj = CQGImpVolAndGreeks('C.TYA'); CQGObj.Startup;
    
    % Define properties
    properties (SetAccess = private, GetAccess = private)
        CQGObj
    end
    
    % Define methods
    methods
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % The constructor
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function obj = CQGImpVolAndGreeks(symbol)
            % Checking correctness of the input.
            error(nargchk(1,1,nargin));
            if isempty(symbol) || ~ischar(symbol) || size(symbol,1)~=1
                error('The symbol must be a character string.');
            end
            % Construct and configure the CQG object
            try
                obj.CQGObj = localCQGConnect();
            catch ME
                error('There was a problem configuring the CQG connection.');
            end
            % attach some user data to the object
            obj.CQGObj.addproperty('UserData');
            obj.CQGObj.UserData.symbol = symbol;
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Start data collection
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function Startup(obj)
            obj.CQGObj.Startup;
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Stop data collection
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function Shutdown(obj)
            localCloseConnection(obj.CQGObj);
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to connect to and configure the CQG object
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function CQGObj = localCQGConnect

% Connect to the CQG engine.
CQGObj = actxserver('CQG.CQGCEL.4');
% Changing APIConfiguration of CQGObj object.
APIConfig = CQGObj.get('APIConfiguration');
APIConfig.set('ReadyStatusCheck', 'rscOff');
APIConfig.set('UsedFromATLClient', true);
APIConfig.set('CollectionsThrowException', false);

% Registering callback event listeners.
CQGObj.registerevent(...
    {'DataConnectionStatusChanged', @localOnDataConnectionStatusChanged;...
    'CELStarted', @localOnCELStarted;...
    'DataError', @localOnDataError;...
    'InstrumentSubscribed', @localOnInstrumentSubscribed;...
    'IncorrectSymbol', @localOnIncorrectSymbol});

end %localCQGConnect

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Handler for the CELStarted event
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function localOnCELStarted(src, evt, varargin) %#ok

try
    % try subscribing to the given symbol.
    fprintf('%s\n%s %s.\n',...
        'CQG Connection has been started.',...
        'Subscribing to', src.UserData.symbol);
    src.NewInstrument(src.UserData.symbol);
    src.UserData.status = 0;
catch ME
    localCloseConnection(src)
    fprintf('%s\n%s %s.\n',...
        'An error occurred trying to start the CQG data collection.',...
        'The specific error was',ME.message);
end
end % localOnCELStarted

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to close the CQG Connection
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function localCloseConnection(src)
% Shutdown the connection if the object has previously been started
try
    if src.IsStarted
        src.Shutdown;
        fprintf('The CQG connection was shut down.\n')
    else
        fprintf('There was no CQC connection to shut down.\n');
    end
catch ME %#ok
    fprintf('There was a problem shutting down the CQC connection.\n');
end
end %localCloseConnection

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to handle data error events
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function localOnDataError(src, evt, handle, eStr, varargin) %#ok

% Let the user know there was an error
fprintf('A data error occured.  The error message is: %s\n', eStr);

end %localOnDataError

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to handle data connection error events
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function localOnDataConnectionStatusChanged(src, evt, newStatus, varargin) %#ok

% Let the know of the connection status change
if (newStatus == 0)
    fprintf('The CQG data connection is up.\n');
else
    fprintf('The CQG data connection is down or delayed.\n');
end
end % localOnDataConnectionStatusChanged

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to handle incorrect symbol error events
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function localOnIncorrectSymbol(src, evt, badSymbol, varargin) %#ok

% Let the user know the symbol was incorrect and shut the connection.
fprintf('The symbol %s is incorrect.\n', badSymbol);
localCloseConnection(src)

end % localOnIncorrectSymbol

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to do the "heavy lifting" when an instrument is successfully
% subscribed to.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function localOnInstrumentSubscribed(src, evt, symbol, Instrument, varargin) %#ok

try
    switch src.UserData.status
        case 0
            % This will occur when the initial instrument is subscribed.
            % Let the user know a subscription has been successful
            switch Instrument.get('InstrumentType')
                case {'itOptionCall','itOptionPut'}
                    %  Subscribe to the option
                    shortName = src.UserData.symbol;
                    src.UserData.symbol = Instrument.get('FullName');
                    fprintf('Option %s (fullname: %s) is being subscribed to.\n',...
                        shortName, src.UserData.symbol);
                    % Subscribe to the underlying future
                    underlyingInstrument = Instrument.get('UnderlyingInstrumentName');
                    fprintf('Subscribing to the underlying future: %s\n',...
                        underlyingInstrument);
                    src.UserData.status = 1;
                    src.NewInstrument(underlyingInstrument);
                otherwise
                    % If the instrument is not an option then stop
                    fprintf('The subscription for this demo must be an option on a future.\n');
                    localCloseConnection(src);
            end
        case 1
            % This will occur when the underlying future is subscribed.
            switch Instrument.get('InstrumentType')
                case 'itFuture'
                    % Get the contract name for calculating the interest rate.
                    Instr = src.get('Instruments').Item(src.UserData.symbol);
                    switch Instr.get('Currency')
                        case 'AUD'
                            currencyInstrument = 'HB';
                        case 'CAD'
                            currencyInstrument = 'KJ';
                        case 'CHF'
                            currencyInstrument = 'QNA';
                        case 'GBP'
                            currencyInstrument = 'QSA';
                        case 'JPY'
                            currencyInstrument = 'EY';
                        case 'SGD'
                            currencyInstrument = 'ZSD';
                        case 'USD'
                            currencyInstrument = 'EDAA';
                        otherwise
                            currencyInstrument = 'QEA';
                    end
                    % Subscribe to the interest rate instrument
                    MonthChar = Instr.get('MonthChar');
                    YearString = Instr.get('YearString');
                    currencyInstrument = sprintf('%s%s%s',...
                        currencyInstrument, MonthChar, YearString(3:4));
                    fprintf('%s %s %s\n',...
                        'Subscribing to',...
                        currencyInstrument,...
                        '(for calculating the interest rate).');
                    src.UserData.status = 2;
                    src.NewInstrument(currencyInstrument);
                otherwise
                    % If the underlying isn't a future then shut down
                    fprint('Black model is only supports an option on a future.');
                    localCloseConnection(src);
            end
        otherwise
            % This will occur when the appropriate currency instrument
            % is subscribed to.
            % Collecting data for calculating IV and Greeks
            qt = Instrument.get('Quotes').Item('qtSettlement');
            if ~qt.get('IsValid')
                fprintf('%s %s %s.\n',...
                    'Settlement of ',...
                    Instrument.get('FullName'),...
                    'is not valid.');
            else
                fprintf('-----------------------------------\n');
                r = (100 - qt.get('Price')) / 100;
                fprintf('Interest Rate = %.4f\n', r);
                
                Instr = src.get('Instruments').Item(src.UserData.symbol);
                qt = Instr.get('Quotes').Item('qtSettlement');
                if ~qt.get('IsValid')
                    fprintf('%s %s %s.\n',...
                        'Settlement of ',...
                        src.UserData.symbol,...
                        'is not valid.');
                else
                    P = qt.get('Price');
                    fprintf('%s %s.\n',...
                        'Option''s settlement = ',...
                        Instr.ToDisplayPrice(P));
                    X = sprintf('%d',Instr.get('Strike'));
                    fprintf('Option''s strike = %s\n', X);
                    X = Instr.FromDisplayPrice(X);
                    sExpirationDate = Instr.get('ExpirationDate');
                    T = round(StringToDate(sExpirationDate) - StringToDate(qt.get('ServerTimeStamp')));
                    fprintf('Option''s DTE = %d\n', T);
                    T = T / 365.25;
                    switch Instr.get('InstrumentType')
                        case 'itOptionCall'
                            PC = 1;
                        otherwise
                            PC = -1;
                    end
                    
                    Instr = Instr.get('UnderlyingInstrument');
                    qt = Instr.get('Quotes').Item('qtSettlement');
                    if ~qt.get('IsValid')
                        fprintf('%s %s %s.\n',...
                            'Settlement of',...
                            Instr.get('FullName'),...
                            'is not valid.');
                    else
                        S = qt.get('Price');
                        fprintf('%s %s\n',...
                            'Underlying instrument''s settlement =',...
                            Instr.ToDisplayPrice(S));
                        fprintf('----------Please wait...-----------\n');
                        %   Calculating IV and Greeks
                        v = localBS_Delta(PC, S, X, P, T, r);
                        fprintf('IV = %.6f.\n', v);
                        fprintf('Delta = %.6f.\n', localBS_Delta(PC, S, X, T, v, r));
                        fprintf('Gamma = %.6f.\n', localBS_Gamma(S, X, T, v, r));
                        fprintf('Theta = %.6f.\n', localBS_Theta(PC, S, X, T, v, r));
                        fprintf('Vega = %.6f.\n', localBS_Vega(S, X, T, v, r));
                        fprintf('Rho = %.6f.\n', localBS_Rho(PC, S, X, T, v, r));
                        fprintf('-----------------------------------\n');
                        % We're done so shut things down
                        localCloseConnection(src)
                    end
                end
            end
    end
catch ME
    localCloseConnection(src)
    fprintf('%s\n%s %s\n',...
        'An error occurred in the function localOnInstrumentSubscribed.',...
        'The specific error was:', ME.message);
end
end % localOnInstrumentSubscribed

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The following are some basic functions for calculating the greeks
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function delta = localBS_Delta(PC,S,X,T,v,r)
d1 = localBS_D1D2(S,X,T,v);
delta = PC*exp(-r*T)*localBS_CND(PC*d1);
end

function gamma = localBS_Gamma(S,X,T,v,r)
d1 = localBS_D1D2(S,X,T,v);
gamma = exp(-(d1^2)/2)/sqrt(2*pi)*exp(-r*T)/(S*v*sqrt(T));
end

function theta = localBS_Theta(PC,S,X,T,v,r)
[d1,d2] = localBS_D1D2(S,X,T,v);
theta = -(S*exp(-((PC*d1)^2)/2)/sqrt(2*pi)*v*exp(-r*T))/(2*sqrt(T)) ...
    + PC*r*S*localBS_CND(PC*d1)*exp(-r*T) ...
    - PC*r*X*exp(-r*T)*localBS_CND(PC*d2);
end

function vega = localBS_Vega(S,X,T,v,r)
d1 = localBS_D1D2(S,X,T,v);
vega = S*sqrt(T)*exp(-(d1^2)/2)/sqrt(2*pi)*exp(-r*T);
end

function rho = localBS_Rho(PC,S,X,T,v,r)
[~, d2] = localBS_D1D2(S,X,T,v);
rho = PC*T*X*exp(-r*T)*localBS_CND(PC*d2);
end

function [d1,d2] = localBS_D1D2(S,X,T,v)
d1 = (log(S/X)+(v^2/2)* T)/(v*sqrt(T));
d2 = d1-v*sqrt(T);
end

function cnd = localBS_CND(x)
cnd = single(0.5*(erf(x)-erf(-inf)));
end
